"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .basesdk import BaseSDK
from speakeasy_client_sdk_python._hooks import HookContext
from speakeasy_client_sdk_python.models import errors, operations
from speakeasy_client_sdk_python.types import BaseModel
import speakeasy_client_sdk_python.utils as utils
from typing import Optional, Union

class Reports(BaseSDK):
    r"""REST APIs for managing reports"""
    
    
    def get_changes_report_signed_url(
        self, *,
        request: Union[operations.GetChangesReportSignedURLRequest, operations.GetChangesReportSignedURLRequestTypedDict],
        server_url: Optional[str] = None,
        timeout_config: Optional[int] = None,
    ) -> operations.GetChangesReportSignedURLResponse:
        r"""Get the signed access url for the change reports for a particular document.

        :param request: The request object to send.
        :param server_url: Override the default server URL for this method
        :param timeout_config: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if server_url is not None:
            base_url = server_url
        
        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.GetChangesReportSignedURLRequest)
        
        req = self.build_request(
            method="GET",
            path="/v1/reports/changes/{documentChecksum}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_config=timeout_config,
        )
        
        http_res = self.do_request(
            hook_ctx=HookContext(operation_id="getChangesReportSignedUrl", oauth2_scopes=[], security_source=self.sdk_configuration.security),
            request=req,
            error_status_codes=["4XX","5XX"],
        )
        
        res = operations.GetChangesReportSignedURLResponse(status_code=http_res.status_code, content_type=http_res.headers.get("Content-Type") or "", raw_response=http_res)
        
        if utils.match_response(http_res, "200", "application/json"):
            res.signed_access = utils.unmarshal_json(http_res.text, Optional[operations.GetChangesReportSignedURLSignedAccess])
        elif utils.match_response(http_res, ["4XX","5XX"], "*"):
            raise errors.SDKError("API error occurred", http_res.status_code, http_res.text, http_res)
        else:
            content_type = http_res.headers.get("Content-Type")
            raise errors.SDKError(f"Unexpected response received (code: {http_res.status_code}, type: {content_type})", http_res.status_code, http_res.text, http_res)

        return res
    
    
    async def get_changes_report_signed_url_async(
        self, *,
        request: Union[operations.GetChangesReportSignedURLRequest, operations.GetChangesReportSignedURLRequestTypedDict],
        server_url: Optional[str] = None,
        timeout_config: Optional[int] = None,
    ) -> operations.GetChangesReportSignedURLResponse:
        r"""Get the signed access url for the change reports for a particular document.

        :param request: The request object to send.
        :param server_url: Override the default server URL for this method
        :param timeout_config: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if server_url is not None:
            base_url = server_url
        
        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.GetChangesReportSignedURLRequest)
        
        req = self.build_request(
            method="GET",
            path="/v1/reports/changes/{documentChecksum}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_config=timeout_config,
        )
        
        http_res = await self.do_request_async(
            hook_ctx=HookContext(operation_id="getChangesReportSignedUrl", oauth2_scopes=[], security_source=self.sdk_configuration.security),
            request=req,
            error_status_codes=["4XX","5XX"],
        )
        
        res = operations.GetChangesReportSignedURLResponse(status_code=http_res.status_code, content_type=http_res.headers.get("Content-Type") or "", raw_response=http_res)
        
        if utils.match_response(http_res, "200", "application/json"):
            res.signed_access = utils.unmarshal_json(http_res.text, Optional[operations.GetChangesReportSignedURLSignedAccess])
        elif utils.match_response(http_res, ["4XX","5XX"], "*"):
            raise errors.SDKError("API error occurred", http_res.status_code, http_res.text, http_res)
        else:
            content_type = http_res.headers.get("Content-Type")
            raise errors.SDKError(f"Unexpected response received (code: {http_res.status_code}, type: {content_type})", http_res.status_code, http_res.text, http_res)

        return res
    
    
    def get_linting_report_signed_url(
        self, *,
        request: Union[operations.GetLintingReportSignedURLRequest, operations.GetLintingReportSignedURLRequestTypedDict],
        server_url: Optional[str] = None,
        timeout_config: Optional[int] = None,
    ) -> operations.GetLintingReportSignedURLResponse:
        r"""Get the signed access url for the linting reports for a particular document.

        :param request: The request object to send.
        :param server_url: Override the default server URL for this method
        :param timeout_config: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if server_url is not None:
            base_url = server_url
        
        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.GetLintingReportSignedURLRequest)
        
        req = self.build_request(
            method="GET",
            path="/v1/reports/linting/{documentChecksum}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_config=timeout_config,
        )
        
        http_res = self.do_request(
            hook_ctx=HookContext(operation_id="getLintingReportSignedUrl", oauth2_scopes=[], security_source=self.sdk_configuration.security),
            request=req,
            error_status_codes=["4XX","5XX"],
        )
        
        res = operations.GetLintingReportSignedURLResponse(status_code=http_res.status_code, content_type=http_res.headers.get("Content-Type") or "", raw_response=http_res)
        
        if utils.match_response(http_res, "200", "application/json"):
            res.signed_access = utils.unmarshal_json(http_res.text, Optional[operations.GetLintingReportSignedURLSignedAccess])
        elif utils.match_response(http_res, ["4XX","5XX"], "*"):
            raise errors.SDKError("API error occurred", http_res.status_code, http_res.text, http_res)
        else:
            content_type = http_res.headers.get("Content-Type")
            raise errors.SDKError(f"Unexpected response received (code: {http_res.status_code}, type: {content_type})", http_res.status_code, http_res.text, http_res)

        return res
    
    
    async def get_linting_report_signed_url_async(
        self, *,
        request: Union[operations.GetLintingReportSignedURLRequest, operations.GetLintingReportSignedURLRequestTypedDict],
        server_url: Optional[str] = None,
        timeout_config: Optional[int] = None,
    ) -> operations.GetLintingReportSignedURLResponse:
        r"""Get the signed access url for the linting reports for a particular document.

        :param request: The request object to send.
        :param server_url: Override the default server URL for this method
        :param timeout_config: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if server_url is not None:
            base_url = server_url
        
        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.GetLintingReportSignedURLRequest)
        
        req = self.build_request(
            method="GET",
            path="/v1/reports/linting/{documentChecksum}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_config=timeout_config,
        )
        
        http_res = await self.do_request_async(
            hook_ctx=HookContext(operation_id="getLintingReportSignedUrl", oauth2_scopes=[], security_source=self.sdk_configuration.security),
            request=req,
            error_status_codes=["4XX","5XX"],
        )
        
        res = operations.GetLintingReportSignedURLResponse(status_code=http_res.status_code, content_type=http_res.headers.get("Content-Type") or "", raw_response=http_res)
        
        if utils.match_response(http_res, "200", "application/json"):
            res.signed_access = utils.unmarshal_json(http_res.text, Optional[operations.GetLintingReportSignedURLSignedAccess])
        elif utils.match_response(http_res, ["4XX","5XX"], "*"):
            raise errors.SDKError("API error occurred", http_res.status_code, http_res.text, http_res)
        else:
            content_type = http_res.headers.get("Content-Type")
            raise errors.SDKError(f"Unexpected response received (code: {http_res.status_code}, type: {content_type})", http_res.status_code, http_res.text, http_res)

        return res
    
    
    def upload_report(
        self, *,
        request: Union[operations.UploadReportRequestBody, operations.UploadReportRequestBodyTypedDict],
        server_url: Optional[str] = None,
        timeout_config: Optional[int] = None,
    ) -> operations.UploadReportResponse:
        r"""Upload a report.

        :param request: The request object to send.
        :param server_url: Override the default server URL for this method
        :param timeout_config: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if server_url is not None:
            base_url = server_url
        
        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.UploadReportRequestBody)
        
        req = self.build_request(
            method="POST",
            path="/v1/reports",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(request, False, False, "multipart", operations.UploadReportRequestBody),
            timeout_config=timeout_config,
        )
        
        http_res = self.do_request(
            hook_ctx=HookContext(operation_id="uploadReport", oauth2_scopes=[], security_source=self.sdk_configuration.security),
            request=req,
            error_status_codes=["4XX","5XX"],
        )
        
        res = operations.UploadReportResponse(status_code=http_res.status_code, content_type=http_res.headers.get("Content-Type") or "", raw_response=http_res)
        
        if utils.match_response(http_res, "200", "application/json"):
            res.uploaded_report = utils.unmarshal_json(http_res.text, Optional[operations.UploadReportUploadedReport])
        elif utils.match_response(http_res, ["4XX","5XX"], "*"):
            raise errors.SDKError("API error occurred", http_res.status_code, http_res.text, http_res)
        else:
            content_type = http_res.headers.get("Content-Type")
            raise errors.SDKError(f"Unexpected response received (code: {http_res.status_code}, type: {content_type})", http_res.status_code, http_res.text, http_res)

        return res
    
    
    async def upload_report_async(
        self, *,
        request: Union[operations.UploadReportRequestBody, operations.UploadReportRequestBodyTypedDict],
        server_url: Optional[str] = None,
        timeout_config: Optional[int] = None,
    ) -> operations.UploadReportResponse:
        r"""Upload a report.

        :param request: The request object to send.
        :param server_url: Override the default server URL for this method
        :param timeout_config: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if server_url is not None:
            base_url = server_url
        
        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.UploadReportRequestBody)
        
        req = self.build_request(
            method="POST",
            path="/v1/reports",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(request, False, False, "multipart", operations.UploadReportRequestBody),
            timeout_config=timeout_config,
        )
        
        http_res = await self.do_request_async(
            hook_ctx=HookContext(operation_id="uploadReport", oauth2_scopes=[], security_source=self.sdk_configuration.security),
            request=req,
            error_status_codes=["4XX","5XX"],
        )
        
        res = operations.UploadReportResponse(status_code=http_res.status_code, content_type=http_res.headers.get("Content-Type") or "", raw_response=http_res)
        
        if utils.match_response(http_res, "200", "application/json"):
            res.uploaded_report = utils.unmarshal_json(http_res.text, Optional[operations.UploadReportUploadedReport])
        elif utils.match_response(http_res, ["4XX","5XX"], "*"):
            raise errors.SDKError("API error occurred", http_res.status_code, http_res.text, http_res)
        else:
            content_type = http_res.headers.get("Content-Type")
            raise errors.SDKError(f"Unexpected response received (code: {http_res.status_code}, type: {content_type})", http_res.status_code, http_res.text, http_res)

        return res
    
